-- Filename:     johnson_counter.vhd
-- Filetype:     VHDL Design Unit
-- Date:         26 feb 2015
-- Update:       -
-- Description:  VHDL Description of a n-stage Johnson Counter
-- Author:       J. op den Brouw
-- State:        Demo
-- Error:        -
-- Version:      1.0
-- Copyright:    (c)2015, De Haagse Hogeschool

-- This VHDL description describes the behaviour of n-stage
-- Johnson counter with spike free active high decoded terminal
-- count output.

-- Using the 1164 std_logic
library ieee;
use ieee.std_logic_1164.all;

-- Tbe port description of the Johnson counter
entity johnson_counter is
    generic (n : integer := 5);
    port (clk :    in std_logic;
          areset : in std_logic;
			 count :  out std_logic_vector (n-1 downto 0);
			 tc :     out std_logic
	      );
end entity johnson_counter;

architecture rtl of johnson_counter is
signal count_int : std_logic_vector (n-1 downto 0);
begin

    -- The Johnson counter itself
    process (clk, areset) is
    begin
        -- The reset is active high
        if areset = '1' then
		      -- Set all counter bits to 0, nice VHDL trick
            count_int <= (others => '0');
        elsif rising_edge(clk) then
		      -- Shift the lot a stage and feed back the last one
				count_int <= count_int(n-2 downto 0) & not count_int(n-1);
		  end if;
	 end process;
		  
    -- The outputs
    count <= count_int;
	 -- Active high when counter is 10..., where the dots shouldt be all zeros.
	 tc <= '1' when count_int(n-1) = '1' and count_int(n-2) = '0' else '0';
end architecture rtl;